@@ -43,6 +43,7 @@ void esd_config_read(void);
 
 /*******************************************************************/
 /* alternate implementations */
+#ifndef HAVE_INET_PTON
 #ifndef HAVE_INET_ATON
 int inet_aton(const char *cp, struct in_addr *inp)
 {
@@ -67,6 +68,7 @@ int inet_aton(const char *cp, struct in_addr *inp)
     return 1;
 }
 #endif
+#endif
 
 /**
  * esd_set_socket_buffers: set buffer lengths on a socket to optimal.
@@ -101,7 +103,7 @@ int esd_set_socket_buffers( int sock, int src_format,
  * @esd: ESD socket
  *
  * Get the stream latency to esound (latency is number of samples
- * at 44.1khz stereo 16 bit - you'll have to adjust if oyur input
+ * at 44.1khz stereo 16 bit - you'll have to adjust if your input
  * sampling rate is less (in bytes per second))
  *
  * Return Value: Latency, in number of samples at 44.1khz stereo 16 bit.
@@ -112,8 +114,8 @@ int esd_get_latency(int esd)
     int proto = ESD_PROTO_LATENCY;
     void (*phandler)(int);
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the necessary information */
@@ -156,14 +158,14 @@ int esd_send_auth( int sock )
     int auth_fd = -1;
     int endian = ESD_ENDIAN_KEY;
     int reply;
-    char *auth_filename = 0;
+    char *auth_filename = NULL;
     unsigned char auth_key[ESD_KEY_LEN];
     char *home = NULL;
     int namelen, retval;
     void (*phandler)(int);
   
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
   
@@ -261,8 +263,8 @@ int esd_lock( int esd ) {
     int ok = 0;
     void (*phandler)(int);
   
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* diagnostic info */
@@ -300,8 +302,8 @@ int esd_unlock( int esd ){
     int ok = 0;
     void (*phandler)(int);
   
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* diagnostic info */
@@ -340,8 +342,8 @@ int esd_standby( int esd )
     int ok = 0;
     void (*phandler)(int);
   
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* diagnostic info */
@@ -379,8 +381,8 @@ int esd_resume( int esd )
     int ok = 0;
     void (*phandler)(int);
   
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* diagnostic info */
@@ -403,7 +405,7 @@ int esd_resume( int esd )
 
 /**
  * esd_connect_tcpip: make a TCPIP connection to ESD
- * @host: Specifies hostname and port to connect to as "hostname:port"
+ * @host: specifies hostname and port to connect to as "hostname:port"
  * Both parts are optional, the default being to connect to localhost on
  * ESD_DEFAULT_PORT.  This default is used if host is NULL.
  *
@@ -423,44 +425,136 @@ esd_connect_tcpip(const char *host)
     char default_host[] = "127.0.0.1";
     char connect_host[64];
     int port = ESD_DEFAULT_PORT;
-    unsigned int host_div = 0;
-    memset (&socket_addr, 0, sizeof (socket_addr));
-    memset (&he, 0, sizeof (he));
-    /* see if we have a remote speaker to play to */
-    espeaker = host;
-    if ( espeaker && *espeaker ) {
-	strncpy(connect_host, espeaker, sizeof(connect_host));
 
-	/* split the espeaker host into host:port */
-	host_div = strcspn( connect_host, ":" );
-	if(host_div > 0 && host_div < strlen(espeaker)) {
-	    connect_host[ host_div ] = '\0';
-	}
-	else if ( host_div == 0)
-		strcpy( connect_host, default_host );
+#if defined (ENABLE_IPV6)
+    if ( have_ipv6() ) {
+       int cnt, i; 
+       char *loc;
+       struct addrinfo hints, *result = NULL, *res;
 
-	connect_host[sizeof(connect_host) - 1] = '\0';
+       memset (&hints, 0, sizeof (hints));
+       hints.ai_socktype = SOCK_STREAM;
+       connect_host[0] = '\0';   
+
+       if ( host ) {
+         cnt = 0;
+         for ( i = 0; i < strlen (host); i++ )
+           if ( host[i] == ':' )
+             cnt++;
+
+         if ( cnt == 1 ) {
+           loc = strchr (host, ':');
+           *loc = '\0';
+           strcpy (connect_host, host);
+           port = atoi (loc + 1);
+         }
+         else {
+           if (( cnt != 0 ) && ((loc = strchr (host, ']')) != NULL )) {
+             *loc = '\0';
+             strcpy ( connect_host, ++host );
+             port = atoi ( loc + 2 );
+           }
+           else 
+             strcpy ( connect_host, host );
+         }
+       }
+       if( port == 0 )
+         port = ESD_DEFAULT_PORT;
+
+       if ( !strlen ( connect_host ) ) 
+         strcpy ( connect_host, "localhost" );
+
+       if ( getaddrinfo ( connect_host, NULL, &hints, &result ) != 0 ) {
+         printf ("Usage:program_name [address][:port]");
+         return (-1);
+       }
+
+       for ( res = result; res ;res = res->ai_next ) {
+         if ( res->ai_family != AF_INET && res->ai_family != AF_INET6 )
+           continue;
+
+         if ( res->ai_family == AF_INET ) 
+           ((struct sockaddr_in *)res->ai_addr)->sin_port = htons(port);
+
+         if ( res->ai_family == AF_INET6 ) 
+           ((struct sockaddr_in6 *)res->ai_addr)->sin6_port = htons(port);
+
+         socket_out = socket ( res->ai_family, SOCK_STREAM, 0 );
+
+         if ( socket_out < 0 ) {
+           fprintf (stderr,"Unable to create TCP socket\n");
+           goto error_out;
+         }
+
+         /* this was borrowed blindly from the Tcl socket stuff */
+         if ( fcntl( socket_out, F_SETFD, FD_CLOEXEC ) < 0 ) {
+           fprintf(stderr,"Unable to set socket to non-blocking\n");
+           goto error_out;
+         }
+
+         if ( setsockopt( socket_out, SOL_SOCKET, SO_REUSEADDR,
+                     &curstate, sizeof(curstate) ) < 0 ) {
+           fprintf(stderr,"Unable to set for a fresh socket\n");
+           goto error_out;
+         }
+
+         if ( connect( socket_out, res->ai_addr, res->ai_addrlen ) != -1 ) 
+           break;
+
+         close ( socket_out );
+       }
+
+       freeaddrinfo ( result );
+
+       if (!res) /* Couldn't connect to any address */
+         return -1;
+    }
+    else
+#endif 
+    {
+       unsigned int host_div = 0;
+       memset (&socket_addr, 0, sizeof (socket_addr));
+       memset (&he, 0, sizeof (he));
+       /* see if we have a remote speaker to play to */
+       espeaker = host;
+       if ( espeaker && *espeaker ) {
+  	 strncpy(connect_host, espeaker, sizeof(connect_host));
+
+	 /* split the espeaker host into host:port */
+	 host_div = strcspn( connect_host, ":" );
+	 if(host_div > 0 && host_div < strlen(espeaker)) {
+	   connect_host[ host_div ] = '\0';
+	 }
+	 else if ( host_div == 0)
+	   strcpy( connect_host, default_host );
+
+   	 connect_host[sizeof(connect_host) - 1] = '\0';
     
-	/* Resolving the host name */
-	if ( ( he = gethostbyname( connect_host ) ) == NULL ) {
-	    fprintf( stderr, "Can\'t resolve host name \"%s\"!\n", 
+	 /* Resolving the host name */
+	 if ( ( he = gethostbyname( connect_host ) ) == NULL ) {
+	   fprintf( stderr, "Can\'t resolve host name \"%s\"!\n", 
 		     connect_host);
-	    return(-1);
-	}
-	memcpy( (struct in_addr *) &socket_addr.sin_addr, he->h_addr,
+	   return(-1);
+	 }
+	 memcpy( (struct in_addr *) &socket_addr.sin_addr, he->h_addr,
 		sizeof( struct in_addr ) );
     
-	/* get port */
-	if ( host_div < strlen( espeaker ) )
-	    port = atoi( espeaker + host_div + 1 );
-	if ( !port ) 
-	    port = ESD_DEFAULT_PORT;
-	/* printf( "(remote) host is %s : %d\n", connect_host, port ); */
-    } else if( !inet_aton( default_host, &socket_addr.sin_addr ) ) {
+	 /* get port */
+	 if ( host_div < strlen( espeaker ) )
+	   port = atoi( espeaker + host_div + 1 );
+	 if ( !port ) 
+	   port = ESD_DEFAULT_PORT;
+	 /* printf( "(remote) host is %s : %d\n", connect_host, port ); */
+    }
+#ifdef HAVE_INET_PTON
+	else if( !inet_pton(AF_INET, default_host, &socket_addr.sin_addr ) ) {
+#else
+	else if( !inet_aton( default_host, &socket_addr.sin_addr ) ) {
+#endif
 	fprintf( stderr, "couldn't convert %s to inet address\n", 
 		 default_host );
-	return -1;
-    }
+	   return -1;
+         }
   
     /* create the socket, and set for non-blocking */
     socket_out = socket( AF_INET, SOCK_STREAM, 0 );
@@ -493,6 +587,7 @@ esd_connect_tcpip(const char *host)
 		  sizeof(struct sockaddr_in) ) < 0 )
 	goto error_out;
 
+    }
     return socket_out;
 
  error_out:
@@ -503,8 +598,6 @@ esd_connect_tcpip(const char *host)
 
 /**
  * esd_connect_unix: make a local UNIX socket connection to ESD
- * @host: Host to connect to - ignored, since UNIX sockets
- * - FIXME - tidy up by removing this parameter.
  *
  * Attempts to make a connection to ESD using local UNIX sockets.
  * Similar to esd_connect_tcpip().
@@ -512,7 +605,7 @@ esd_connect_tcpip(const char *host)
  * Return Value: -1 on error, else a socket number connected to ESD.
  */
 static int
-esd_connect_unix(const char *host)
+esd_connect_unix(void)
 {
     struct sockaddr_un socket_unix;
     int socket_out = -1;
@@ -558,7 +651,7 @@ esd_connect_unix(const char *host)
 static int is_host_local(const char* host) 
 {
     char hnbuf[256]="";
-    if (!host || (*host)) return 1;
+    if (!host || !*host) return 1;
 
     gethostname(hnbuf, sizeof(hnbuf));
     return (!strcasecmp(host,hnbuf) || !strcasecmp(host, "localhost"));
@@ -574,15 +667,15 @@ static int is_host_local(const char* host)
  * host specified by the $ESPEAKER environment variable if @host is NULL,
  * or localhost if $ESPEAKER not set.
  *
- * Will attempt to connect by UNIX sockets if the host is localhost, and by
+ * Will attempt to connect via UNIX sockets if the host is localhost, and by
  * TCPIP otherwise, or if UNIX sockets fail.
  *
- * If neither of these connection methods succeeds, and we are attempting to
+ * If neither of these connection methods succeed, and we are attempting to
  * contact the localhost, will attempt to spawn a local copy of ESD (unless
  * configured not to in esd.conf), and will then try to connect to that
  * using UNIX sockets.
  *
- * Once a connection is created, attempts to give ESD the neccessary
+ * Once a connection is created, we attempt to give ESD the necessary
  * authorisation keys to do things - only if this succeeds will the socket
  * be given to the caller.
  *
@@ -621,7 +714,7 @@ int esd_open_sound( const char *host )
 	    use_unix = 1;
     }
     if ( use_unix )
-	socket_out = esd_connect_unix( NULL );
+	socket_out = esd_connect_unix();
     if ( socket_out >= 0 ) goto finish_connect;
 
     socket_out = esd_connect_tcpip( host );
@@ -672,17 +765,17 @@ int esd_open_sound( const char *host )
 	} else {
 	    int estat;
 
-	    waitpid(childpid, &estat, 0);
+	    while ((waitpid (childpid, &estat, 0)== -1) && (errno == EINTR));
 	}
 
-	/* Wait for for spawning to happen.  Time taken is system and load
+	/* Wait for spawning to happen.  Time taken is system and load
 	 * dependent, so read from config file.
 	 */
 	FD_ZERO (&fdset);
 	FD_SET (esd_pipe[0], &fdset);
 	
-	timeout.tv_sec = 0;
-	timeout.tv_usec = esd_spawn_wait_ms * 1000;
+	timeout.tv_sec = (esd_spawn_wait_ms * 1000) / 1000000;
+	timeout.tv_usec = (esd_spawn_wait_ms * 1000) % 1000000;
 	
 	ret = select (esd_pipe[0] + 1, &fdset, NULL, NULL, &timeout);
 	
@@ -691,7 +784,7 @@ int esd_open_sound( const char *host )
 	    ret = read (esd_pipe[0], &c, 1);
 
 	    if (ret == 1) {
-		socket_out = esd_connect_unix(host);
+		socket_out = esd_connect_unix();
 		if (socket_out < 0)
 		    socket_out = esd_connect_tcpip(host);
 	    }
@@ -716,7 +809,7 @@ int esd_open_sound( const char *host )
  * @format: data format for this stream
  * @rate: sample rate for this stream
  * @host: host to connect to, as for esd_open_sound().
- * @name: name by which to identify this stream to ESD.  (Use NULL if you
+ * @name: name used to identify this stream to ESD.  (Use NULL if you
  * don't care what name you're given - but its generally more useful to give
  * something helpful, such as your process name and id.)
  *
@@ -745,8 +838,8 @@ int esd_play_stream( esd_format_t format, int rate,
     else
 	name_buf[ 0 ] = '\0';
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the audio format information */
@@ -788,7 +881,7 @@ int esd_play_stream( esd_format_t format, int rate,
  * @format: data format for this stream
  * @rate: sample rate for this stream
  * @host: host to connect to, as for esd_open_sound().
- * @name: name by which to identify this stream to ESD.
+ * @name: name used to to identify this stream to ESD.
  *
  * Attempts to create a connection to an ESD, using esd_play_stream(), and if
  * this fails falls back to trying to contact the soundcard directly.
@@ -832,7 +925,7 @@ int esd_play_stream_fallback( esd_format_t format, int rate,
  * @format: data format for this stream
  * @rate: sample rate for this stream
  * @host: host to connect to, as for esd_open_sound().
- * @name: name by which identify this stream to ESD.
+ * @name: name used to identify this stream to ESD.
  *
  * Creates a new connection to ESD, using esd_open_sound(), and sets it up
  * for monitoring the output from the ESD.
@@ -859,8 +952,8 @@ int esd_monitor_stream( esd_format_t format, int rate,
     else
 	name_buf[ 0 ] = '\0';
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the audio format information */
@@ -902,7 +995,7 @@ int esd_monitor_stream( esd_format_t format, int rate,
  * @format: data format for this stream
  * @rate: sample rate for this stream
  * @host: host to connect to, as for esd_open_sound().
- * @name: name by which identify this stream to ESD.
+ * @name: name used to identify this stream to ESD.
  *
  * Creates a new connection to ESD, using esd_open_sound(), and sets it up
  * for filtering the output from the ESD.
@@ -910,7 +1003,7 @@ int esd_monitor_stream( esd_format_t format, int rate,
  * Reading from the stream will give a block of audio data, which is the
  * mixed output of the ESD formatted as specified by the function parameters.
  * The filter is free to process this data as it likes, but must then write
- * an indentically sized block of data back to the stream.  The data so
+ * an identically sized block of data back to the stream.  The data so
  * returned is played by the ESD (possibly after applying more filters to
  * it.)
  *
@@ -940,8 +1033,8 @@ int esd_filter_stream( esd_format_t format, int rate,
     else
 	name_buf[ 0 ] = '\0';
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the audio format information */
@@ -983,7 +1076,7 @@ int esd_filter_stream( esd_format_t format, int rate,
  * @format: data format for this stream
  * @rate: sample rate for this stream
  * @host: host to connect to, as for esd_open_sound().
- * @name: name by which identify this stream to ESD.
+ * @name: name used to identify this stream to ESD.
  *
  * Creates a new connection to ESD, using esd_open_sound(), and sets it up
  * for recording data from the soundcard via the ESD.
@@ -1010,8 +1103,8 @@ int esd_record_stream( esd_format_t format, int rate,
     else
 	name_buf[ 0 ] = '\0';
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the audio format information */
@@ -1053,7 +1146,7 @@ int esd_record_stream( esd_format_t format, int rate,
  * @format: data format for this stream
  * @rate: sample rate for this stream
  * @host: host to connect to, as for esd_open_sound().
- * @name: name by which to identify this stream to ESD.
+ * @name: name used to identify this stream to ESD.
  *
  * Attempts to create a connection to an ESD, using esd_record_stream(), and if
  * this fails falls back to trying to contact the soundcard directly.
@@ -1095,8 +1188,18 @@ int esd_record_stream_fallback( esd_format_t format, int rate,
     return socket_out;
 }
 
-/*******************************************************************/
-/* cache a sample in the server returns sample id, <= 0 is error */
+/**
+ * esd_sample_cache: Cache sample in server
+ * @esd: server to cache the data in
+ * @format: data format for this stream 
+ * @rate: sampling rate of the sample
+ * @size: size of the sample
+ * @name: name of the sample being cached
+ *
+ * Caches a sample in the server.
+ * 
+ * Return value: ID of the sample cached, or -1 on error.
+ */
 int esd_sample_cache( int esd, esd_format_t format, const int rate, 
 		      const int size, const char *name )
 {
@@ -1113,8 +1216,8 @@ int esd_sample_cache( int esd, esd_format_t format, const int rate,
     /* printf( "caching sample: %s (%d) - %ld bytes\n", 
        name_buf, esd, size ); */
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the necessary information */
@@ -1168,8 +1271,8 @@ int esd_confirm_sample_cache( int esd )
     int id = 0;
     void (*phandler)(int);
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* get the sample id back from the server */
@@ -1198,8 +1301,8 @@ int esd_sample_getid( int esd, const char *name)
     char namebuf[ESD_NAME_MAX];
     void (*phandler)(int);
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     if ( write( esd, &proto, sizeof(proto) ) != sizeof(proto) ) {
@@ -1240,7 +1343,7 @@ int esd_sample_getid( int esd, const char *name)
 }
 
 /*******************************************************************/
-/* uncache a sample in the server */
+/* remove a sample from the cache in the server */
 int esd_sample_free( int esd, int sample )
 {
     int id;
@@ -1249,8 +1352,8 @@ int esd_sample_free( int esd, int sample )
 
     /* printf( "freeing sample (%d) - <%d>\n", esd, sample ); */
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the necessary information */
@@ -1282,7 +1385,7 @@ int esd_sample_free( int esd, int sample )
 }
 
 /*******************************************************************/
-/* uncache a sample in the server */
+/* play a sample */
 int esd_sample_play( int esd, int sample )
 {
     int is_ok;
@@ -1291,8 +1394,8 @@ int esd_sample_play( int esd, int sample )
 
     /* printf( "playing sample (%d) - <%d>\n", esd, sample ); */
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the necessary information */
@@ -1335,8 +1438,8 @@ int esd_sample_loop( int esd, int sample )
 
     /* printf( "looping sample (%d) - <%d>\n", esd, sample ); */
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the necessary information */
@@ -1377,8 +1480,50 @@ int esd_sample_stop( int esd, int sample )
 
     /* printf( "stopping sample (%d) - <%d>\n", esd, sample ); */
 
-/* this is unavoidable - incase ESD "disappears" (ie the socket conn dies) */
-/* we need to catch SIGPIPE to avoid the default handler form giving us */
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
+/* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
+    phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
+    /* send the necessary information */
+    if ( write( esd, &proto, sizeof(proto) ) != sizeof(proto) ) {
+	signal( SIGPIPE, phandler ); 
+	return -1;
+    }
+    if ( write( esd, &sample, sizeof(sample) ) != sizeof(sample) ) {
+	signal( SIGPIPE, phandler ); 
+	return -1;
+    }
+    /* fsync( esd ); */
+
+    /* get the sample id back from the server */
+    if ( read( esd, &is_ok, sizeof(is_ok) ) != sizeof(is_ok) ) {
+	signal( SIGPIPE, phandler ); 
+	return -1;
+    }
+
+    /* diagnostic info */
+    /*
+      if ( getenv( "ESDBG" ) )
+      printf( "esound stopping sample\n" );
+    */
+
+    /* return the id to the client (0 = error, 1 = ok) */
+    signal( SIGPIPE, phandler ); 
+    return is_ok;
+}
+
+/*******************************************************************/
+/* stop a looping sample in the server */
+int esd_sample_kill( int esd, int sample )
+{
+    int is_ok;
+    int proto = ESD_PROTO_SAMPLE_KILL;
+    void (*phandler)(int);
+
+    /* printf( "killing sample (%d) - <%d>\n", esd, sample ); */
+
+/* this is unavoidable - in case ESD "disappears" (ie the socket conn dies) */
+/* we need to catch SIGPIPE to avoid the default handler giving us */
 /* a bad day - ignore the SIGPIPE, then make sure to catch all errors */
     phandler = signal( SIGPIPE, dummy_signal );    /* for closed esd conns */
     /* send the necessary information */
@@ -1421,3 +1566,4 @@ int esd_close( int esd )
 
     return close( esd );
 }
+
