@@ -5,6 +5,7 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <string.h>
 
 int main(int argc, char **argv)
 {
@@ -54,7 +55,7 @@ int main(int argc, char **argv)
     }
 
     format = bits | channels | mode | func;
-    printf( "opening socket, format = 0x%08x at %d Hz\n", 
+    fprintf( stderr, "opening socket, format = 0x%08x at %d Hz\n", 
 	    format, rate );
    
     /* sock = esd_record_stream( format, rate ); */
